#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for `myratings` package."""


import unittest
from click.testing import CliRunner

from myratings import myratings
from myratings import cli
from myratings.codechef import obtainRating as ccor
from myratings.codeforces import obtainRating as cfor
from myratings.hackerearth import obtainRating as hkor


class TestMyratings(unittest.TestCase):
    """Tests for `myratings` package."""
    
    def test_codechef_on_proper_input(self):
        """Tests codechef's obtainRating method behaviour on proper input."""        
        self.assertTrue(ccor("http://www.codechef.com/users/kgarun") != "NA")
    
    def test_codechef_on_improper_input(self):
        """Tests codechef's obtainRating method behaviour on improper input."""
        self.assertTrue(ccor("http://www.codechef.com/users/abc12sffnkk") == "NA")
    
    def test_codeforces_on_proper_input(self):
        """Tests codeforces's obtainRating method behaviour on proper input."""        
        self.assertTrue(cfor("http://www.codeforces.com/profile/kgarun50") != "NA")
    
    def test_codeforces_on_improper_input(self):
         """Tests codeforces's obtainRating method behaviour on improper input."""
         self.assertTrue(cfor("http://www.codeforces.com/profile/kgarunaffgwgs50") == "NA")
    
    def test_hackerearth_on_proper_input(self):
        """Tests hackerearth's obtainRating method behaviour on proper input."""        
        self.assertTrue(hkor("https://www.hackerearth.com/@kgarun50") != "NA")
    
    def test_hackerearth_on_improper_input(self):
        """Tests hackerearth's obtainRating method behaviour on improper input.""" 
        self.assertTrue(hkor("https://www.hackerearth.com/@kgarundgdhhhhh50") == "NA")

    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'myratings.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output



from .scrape import *


def obtainRating(url):
    """Obtains User Rating From Codechef Website.
    Arguments:
        url = Url of the User Profile
    Returns:
        Actual ratings if successfully retrieved, NA otherwise
    """

    soup = getSoup(url)
    rating = "NA"

    try:
        ratingDiv = soup.findAll("div", {"class": "rating-number"})
        rating = ratingDiv[0].text
        if rating == "NA":
            raise Exception("Rating not Found!!")
        return rating
    except Exception as e:
        #print("Cannot Obtain Ratings From CodeChef")
        #print("Reason :" + str(e))
        return rating



"""Displays the Rating Summary of the User from various CP Sites.
    Mode 1:
        In this mode user handles and ratings are stored in the disk so that
        the user need not enter his/her handle everytime and also provides
        comparison between old and new ratings.It the default mode.
    Mode 2:
        If it is not possible to maintain log, users can still view their
        ratings by simply selecting competitive programming site option number
        and entering handle.
"""

import os
import errno
import fileinput
from .judges import *
from texttable import Texttable


def manageLog():
    """ Manages the handle folder (for persistence data where the id of the
    user in various Competitive Programming site is stored.
    Returns:
        True = Upon successfully creating handle folder and subfiles
        False = Otherwise
    """
    configureReq = False
    if not os.path.isdir("handle"):
        configureReq = True
        try:
            os.makedirs("handle")  # Creating handle Folder

            with open('handle/log.txt',
                      'w') as log:  # Creating and Initializing Log File
                for judge in judgesList:
                    log.write(judge + " " + "0\n")

        except OSError as e:
            print("Cannot Manage Handle Folder\nReason: ", end="")
            if e.errno == errno.EACCES:
                print("Permission denied")
            elif e.errno == errno.ENOSPC:
                print("No space left on device")
            elif e.errno == errno.EROFS:
                print("Read-only file system")
            else:
                print(str(e))
            return False

    for judge in judgesList:  # Creating Log file for each judge
        filename = judge + '.txt'
        filepath = "handle/" + filename
        if not os.path.exists(filepath):
            open(filepath, 'w').close()

    if configureReq:
        configure()

    return True


def configure():
    """Helps to set the Userid and Judge Preferences.
    """

    for judge in judgesList:
        print("Do you want to add " + judge + "?? (y/n) ", end="")
        response = input()
        if response.lower() == "y":
            with open("handle/" + judge + ".txt", 'w') as log:
                print("Enter your handle: ", end="")
                response = input()
                log.write(response + " NA")  # Updating user Handle

            for line in fileinput.input(
                    "handle/log.txt", inplace=1):  # Updating Log File
                print(line.replace(judge + " 0\n", judge + " 1\n"), end="")


def getRatings():
    """Obtains the Ratings summary of the User and Updates respective Log File.
    Returns:
        rating = a list of lists(site,oldrating,newrating)
    """

    rating = []
    userHandleError = []

    with open('handle/log.txt', 'r') as log:
        for line in log:
            site, preference = line[:-1].split(" ")# Splitting Site and Preference
            if preference == "1":
                userId, oldRating, newRating = None, None, None
                with open('handle/' + site + '.txt', 'r') as siteHandle:
                    for info in siteHandle:
                        userId, oldRating = info.split(" ")  # Obtaining UserId and Ratings

                reqUrl = url[site] + userId  # Constructing Request URL
                newRating = obtainRatings(site, reqUrl)  # Obtaining Latest Rating
                if(newRating == "NA"):
                    userHandleError.append(site)
                else:
                    rating.append([site, oldRating, newRating])

                if newRating != "NA":  # Updating LogFile
                    if oldRating == "NA" or int(oldRating) != int(newRating):
                        with open('handle/' + site + '.txt','w') as siteHandle:
                            siteHandle.write(userId + " " + str(newRating))

    if len(userHandleError) > 0:
        print("\nPlease enter correct user handle of the following site(s):")
        for errhandle in userHandleError:
            print("   ==> "+errhandle)        

        print("\nUse following command to config handle:\n myratings config\n\n")

    return rating


def printRatingSummary(ratings):
    """Prints rating Summary.
    Arguments:
        rating =  a list of lists(site,oldrating,newrating)
    """

    summary = []

    if len(ratings) > 0:
        table = Texttable()
        headers=['Site', 'OldRating','NewRating','Changes']
        summary.append(headers)

        for rating in ratings:
            sublist = []
            site = rating[0]
            oldRating = "NA"
            newRating = "NA"
            change = "NA"

            if rating[1] != "NA":
                oldRating = int(rating[1])
            if rating[2] != "NA":
                newRating = int(rating[2])
            if oldRating != "NA" and newRating != "NA":
                change = newRating - oldRating

            sublist.append(site)
            sublist.append(oldRating)
            sublist.append(newRating)
            sublist.append(change)
            summary.append(sublist)

        table.add_rows(summary)
        print(table.draw())
        print()


def onlineRatings():
    """Displays the Latest rating of the user.No log file is maintained.
    """

    counter = 1
    ptr = None
    print("Please Select one of the following options")
    for judge in judgesList:
        print(str(counter) + " ==> " + judge)
        counter += 1

    while True:
        print("Select any option.Press 'Q' to quit")
        ptr = input()  # Inputting Option

        try:
            ptr = int(ptr)
            if ptr < 1 or ptr >= counter:  # Doesnt fit in the limit
                raise Exception("Quitting....")
            else:
                site = judgesList[ptr - 1]
                reqUrl = url[site]
                print("Enter Your " + site + " Handle: ", end="")
                handle = str(input())
                reqUrl += handle
                print("Your " + site + " ratings : " +
                      obtainRatings(site, reqUrl))

        except Exception as e:
            print("Wish you High Ratings!!")
            break  # Breaks infinite loop


def main():
    """ Main function of the Module. """

    args = os.sys.argv

    if len(args) > 1:
        if args[1].lower() == "config":
            configure() # Calls method that configure user handle
        else:
            print("Usage:\n myratings (or) \n myratings config ")
            os.sys.exit()

    if manageLog() is True:  # Mode 1
        ratings = getRatings()
        printRatingSummary(ratings)
    else:  # Mode 2
        print("Still You can view your ratings :)")
        onlineRatings()


if __name__ == "__main__":
    main()




from .codechef import obtainRating as ccfor
from .codeforces import obtainRating as cfsor
from .hackerearth import obtainRating as heor 

judgesList = ['hackerearth', 'codechef', 'codeforces']

url = {
    'hackerearth': r'https://www.hackerearth.com/@',
    'codechef': r'https://www.codechef.com/users/',
    'codeforces': r'http://codeforces.com/profile/',
}


def obtainRatings(site, reqUrl):
    """ Obtains Rating From Corresponding Site.
    Arguments:
        site = Name of the Site
        reqUrl = Url of the user profile
    Returns:
        Actual ratings if successfully retrieved, NA otherwise
    """

    rating = "NA"
    if site == "codechef":
        rating = ccfor(reqUrl)
    elif site == "codeforces":
        rating = cfsor(reqUrl)
    elif site == "hackerearth":
        rating = heor(reqUrl)
    return rating



from .scrape import *
import re


def obtainRating(url):
    """Obtains User Rating From Codeforces Website.
    Arguments:
        url = Url of the User Profile
    Returns:
        Actual ratings if successfully retrieved, NA otherwise
    """

    soup = getSoup(url)
    rating = "NA"

    try:
        userRatingInfo = soup.findAll("span", {"class": re.compile("^user-")})
        for info in userRatingInfo:
            try:
                rating = int(info.text)
                break
            except ValueError:
                continue
        if rating == "NA":
            raise Exception("Rating not Found!!")
        return str(rating)
    except Exception as e:
        #print("Cannot Obtain Ratings From CodeForces")
        #print("Reason :" + str(e))
        return rating



import requests
from bs4 import BeautifulSoup


def getSoup(url):
    """Returns BeatifulSoup object of the given URL.
    Arguments:
        url = URL of the corresponding webpage
    Returns:
        BeatifulSoup object of the given URL
    """
    try:
        r = requests.get(
            url,
            headers={
                "referer":
                url,
                'User-Agent':
                'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) \
                 AppleWebKit/537.36 (KHTML, like Gecko) \
                 Chrome/39.0.2171.95 Safari/537.36'
            })
        soup = BeautifulSoup(r.content, 'html.parser')
        return soup
    except Exception as e:
        print(str(e))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
	'requests>=2.18.4',
	'bs4',
    'texttable',
]

setup_requirements = [
    # TODO(kgarun): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='myratings',
    version='1.1.2',
    description="Displays ratings of the user from various competitive programming sites in the terminal !!",
    long_description=readme + '\n\n' + history,
    author="Arun KG",
    author_email='kgarun50@gmail.com',
    url='https://github.com/kgarun/myratings',
    packages=['myratings',],
    entry_points={
        'console_scripts': [
            'myratings=myratings.myratings:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='myratings competitive programming cp terminal rating ratings',
	python_requires='>=3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
		'Topic :: Terminals',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.5',	
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)
